type TopicEventHandler<T> = (payload: T) => unknown | Promise<unknown>;
type TopicSubscriptionType = "before" | "on" | "after";
type Transformer<T, K> = (payload: T) => K;
type PipeCheck<T> = (payload: T) => boolean;
interface TopicSubscription<T> {
    /** Call this function to remove this subscription */
    unsubscribe: () => void;
    /** The original handler function passed in for this subscription */
    handler: TopicEventHandler<T>;
    /** The type of subscription (one of "before" "on" or "after") */
    type: TopicSubscriptionType;
}
/**
 * A simple pub-sub like topic system for easy event publishing and listening
 */
declare class Topic<T> {
    private beforeListeners;
    private onListeners;
    private afterListeners;
    private removeSubscription;
    private createSubscription;
    /**
     * Register an event handler to run before normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    before: (handler: TopicEventHandler<T>) => TopicSubscription<T>;
    /**
     * Register an event handler
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    subscribe: (handler: TopicEventHandler<T>) => TopicSubscription<T>;
    /**
     * Register an event handler to run after normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    after: (handler: TopicEventHandler<T>) => TopicSubscription<T>;
    /**
     * Publish data and an optional event name ot the topic
     * @param payload Data to publish for this topic
     */
    publish: (payload: T) => void;
    /**
     * Automatically convert events from this topic and publish to another topic
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing
     */
    pipe: <K>(topic: Topic<K>, transformer: Transformer<T, K>) => void;
    /**
     * Automatically convert events from this topic and publish to another topic.
     * If the transformer returns void, the pipe will short-circuit instead.
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing or returns void to skip publishing
     */
    optionalPipe: <K>(topic: Topic<K>, check: PipeCheck<T>, transformer: Transformer<T, K>) => void;
}

declare const CAMPER_DEFAULT_AVATAR = "https://cdn.astral.camp/general-assets/default-avatar.png";
declare const CAMPER_DEFAULT_COLOR = "#FFFFFF";
declare const CAMPER_HANDLE_REGEX: RegExp;
declare const CAMPER_DISPLAY_NAME_MAX_LENGTH = 32;
declare const CAMPER_TAGLINE_MAX_LENGTH = 120;
declare const STREAMING_SERVICES: readonly ["twitch"];
declare const CHAT_SERVICES: readonly ["relay", "discord", "twitch"];
declare const POST_SERVICES: readonly ["relay", "mastodon", "prose", "vibe-check"];
declare const MEDIA_ATTACHMENT_TYPES: string[];

type StreamingService = (typeof STREAMING_SERVICES)[number];
type ChatService = (typeof CHAT_SERVICES)[number];
type MediaAttachmentType = (typeof MEDIA_ATTACHMENT_TYPES)[number];
type PostService = (typeof POST_SERVICES)[number];

type RelayID<T extends RelayObject> = `${T}_${string}`;
declare const RELAY_OBJECT_TYPES: readonly ["chat", "post", "concoction", "plant", "vibecheck", "camper", "character", "token", "letter", "starboard", "portent", "note", "shortlink", "event", "socket"];
type RelayObject = (typeof RELAY_OBJECT_TYPES)[number];

interface APICamperServiceConnections {
    mastodon?: {
        handle: string;
        displayName: string;
        id: string;
    };
    twitch?: {
        handle: string;
        displayName: string;
        color: string;
        id: string;
        creatorMode: boolean;
    };
    discord?: {
        handle: string;
        displayName: string;
        id: string;
    };
    prose?: {
        handle: string;
    };
}
/**
 * A small representation of a Camper
 */
interface APICamper {
    id: RelayID<"camper">;
    handle: string;
    name: string;
    pronouns: string | null;
    avatar: string;
    color: string;
    services: APICamperServiceConnections;
}
interface APIMediaAttachment {
    type: MediaAttachmentType;
    url: string;
    description: string | null;
    warning: string | null;
    credit: string | null;
    mimeType: string | null;
}
/**
 * Represents the custom camper event data sent over the wire
 * to the client. Omits camper info as the event is scoped to
 * the relevant camper's client
 */
type APICustomCamperEvent<E extends string = string, T = unknown> = {
    eventName: E;
    eventData: T;
};

interface APIRelayMessageMetadata {
    twitch: {
        subscriber: boolean;
        vip: boolean;
    } | null;
    discord: {
        messageId: string;
        channelId: string;
        parentChannelId: string | null;
        isThread: boolean;
    } | null;
}
interface APIRelayMessage {
    id: RelayID<"chat">;
    camper: APICamper | null;
    content: string;
    richContent: string;
    author: string;
    avatar: string;
    color: string;
    source: ChatService;
    sourceChannel: string;
    sourceId: string;
    replyTo: RelayID<"chat"> | null;
    createdAt: string;
    attachments: APIMediaAttachment[];
    metadata: APIRelayMessageMetadata;
}

interface APIMailLetter {
    id: RelayID<"letter">;
    from: APICamper;
    content: string;
    createdAt: string;
    read: boolean;
}

declare const NOTE_VISIBILITY_OPTIONS: readonly ["public", "private"];
declare const NOTE_CATEGORY_OPTIONS: readonly ["game", "game:botany", "game:chemistry", "game:economy", "game:badge", "game:item", "game:portent", "social", "social:mastodon", "social:prose", "social:twitch", "social:discord", "camp", "camp:mail", "camp:vibe-check", "camp:shortlink", "camp:camper"];

type NoteVisibility = (typeof NOTE_VISIBILITY_OPTIONS)[number];
type NoteCategory = (typeof NOTE_CATEGORY_OPTIONS)[number];

interface APINote {
    id: RelayID<"note">;
    visibility: NoteVisibility;
    content: string;
    category: NoteCategory;
    createdAt: string;
}

interface APIRelayPost {
    id: RelayID<"post">;
    camper: APICamper | null;
    source: PostService;
    sourceChannel: string;
    sourceId: string;
    sourceUrl: string;
    title: string | null;
    summary: string | null;
    author: string;
    published: string;
    content: string;
    tags: string[];
    attachments: APIMediaAttachment[];
    createdAt: string;
    updatedAt: string | null;
}

/**
 * Represents a Camper's profile and character information
 * Does not include settings or private information
 */
interface APICamperProfile {
    camperId: RelayID<"camper">;
    handle: string;
    name: string;
    pronouns: string;
    avatar: string;
    color: string;
    tagline: string;
    website: string;
    services: APICamperServiceConnections;
}

interface APIShortlink {
    alias: string;
    href: string;
}

interface BaseAPIStreamEvent {
    /** The service this came from */
    source: StreamingService;
    /** The channel ID of the streamer (ID, not handle) */
    sourceChannel: string;
    /** The Camper who is broadcasting */
    broadcastingCamper: APICamper;
}
interface APIStreamLiveEvent extends BaseAPIStreamEvent {
    /**
     * An ISO 8601 timestamp of when the stream went live
     */
    timestamp: string;
}
interface APIStreamRequestEvent extends BaseAPIStreamEvent {
    /** The title of the request */
    request: string;
    /**
     * The message associated with the request
     * If no message is supplied, this will be an empty string
     */
    message: string;
    /**
     * The computed name of the user who made the request.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who made the request.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who made the request.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** If set, represents the Camper who made the request */
    camper: APICamper | null;
}
interface APIStreamFollowEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who followed the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who followed the broadcaster.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who followed.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** If set, represents the Camper who followed the broadcaster */
    camper: APICamper | null;
}
interface APIStreamSubEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who subscribed to the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who subscribed.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who subscribed.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /**
     * The message associated with the subcription,
     * or null if no message was provided
     */
    message: string | null;
    /** The duration of the subscription in months */
    months: number;
    /** If set, represents the Camper who subscribed to the broadcaster */
    camper: APICamper | null;
}
interface APIStreamGiftedSubEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who gifted a subscription.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     * If the gift was anonymous, this will be a placeholder name
     * such as "A Generous Soul"
     */
    gifterName: string;
    /** If set, represents the Camper who gifted the subscription */
    gifterCamper: APICamper | null;
    /**
     * The computed avatar of the user who gifted the subscription.
     * If there was no avatar to use, this will be null
     */
    gifterAvatar: string | null;
    /**
     * The computed color associated with the user who gifted the subscription.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    gifterColor: string;
    /**
     * The computed name of the user who received the gifted subscription.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    recipientName: string;
    /** If set, represents the Camper who received the gift */
    recipientCamper: APICamper | null;
    /**
     * The computed avatar of the user who received the gifted subscription.
     * If there was no avatar to use, this will be null
     */
    recipientAvatar: string | null;
    /**
     * The computed color associated with the user who received the gifted subscription.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    recipientColor: string;
}
interface APIStreamMassGiftedSubEvent extends BaseAPIStreamEvent {
    /** The number of subscriptions gifted */
    count: number;
    /**
     * The computed name of the user who gifted a subscription.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     * If the gift was anonymous, this will be a placeholder name
     * such as "A Generous Soul"
     */
    username: string;
    /**
     * The computed avatar of the user who gifted the subscriptions.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who gifted the subscriptions.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** If set, represents the Camper who gifted the subscriptions */
    camper: APICamper | null;
}
interface APIStreamRaidEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who raided the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who raided.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who raided.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** The number of viewers the raider brought along */
    count: number;
    /** If set, represents the Camper who raided the broadcaster */
    camper: APICamper | null;
}
interface APIStreamCheerEvent extends BaseAPIStreamEvent {
    /**
     * The computed name of the user who cheered the broadcaster.
     * If this was from a known Camper, this will be their preferred Camp name
     * Otherwise, it will be derived from the upstream event
     */
    username: string;
    /**
     * The computed avatar of the user who cheered.
     * If there was no avatar to use, this will be null
     */
    avatar: string | null;
    /**
     * The computed color associated with the user who cheered.
     * If this is a camper, this will be their favorite color setting
     * Othherwise, it will be derived from the upstream event
     */
    color: string;
    /** The amount of currency cheered with (Bits for Twitch) */
    amount: number;
    /** If set, represents the Camper who cheered the broadcaster */
    camper: APICamper | null;
}

/**
 * Error thrown when the Relay API returns an error response
 * Contains the HTTP status code and status text for debugging
 */
declare class RelayAPIError extends Error {
    /** The HTTP status code returned by the API */
    status: number;
    /** The HTTP status text returned by the API */
    statusText: string;
    constructor(status: number, statusText: string);
}

/**
 * Configuration options for the Relay client
 *
 * @example
 * ```typescript
 * const config: Partial<RelayClientConfig> = {
 *   socketHost: "localhost",
 *   socketPort: 8080,
 *   socketProtocol: "ws",
 *   httpHost: "localhost",
 *   httpPort: 3000,
 *   httpProtocol: "http"
 * };
 * ```
 */
interface RelayClientConfig {
    /** The hostname of the Relay WebSocket server (default: "relay-ws.astral.camp") */
    socketHost: string;
    /** The port of the Relay WebSocket server (default: 443) */
    socketPort: number;
    /** The protocol of the Relay WebSocket server (default: "wss") */
    socketProtocol: "ws" | "wss";
    /** The path of the Relay WebSocket server (default: "/ws") */
    socketPath: string;
    /** Additional query parameters for the WebSocket connection (default: "") */
    socketQuery: string;
    /** The hostname of the Relay HTTP server (default: "relay.astral.camp") */
    httpHost: string;
    /** The port of the Relay HTTP server (default: 443) */
    httpPort: number;
    /** The protocol of the Relay HTTP server (default: "https") */
    httpProtocol: "http" | "https";
}
/**
 * Client API for the Stardust Relay
 *
 * @example Usage
 * ```typescript
 * import RelayClient from '@stardust-relay/client';
 *
 * const client = new RelayClient("your-auth-token");
 *
 * // Subscribe to chat messages
 * client.Events.Chat.new.subscribe((message) => {
 *   console.log(message.author, message.content);
 * });
 *
 * // Fetch a camper's profile
 * const profile = await client.getProfile('username');
 *
 * // Send a custom event
 * await client.emit("namespace:event", { key: "value" });
 * ```
 */
declare class RelayClient {
    /** The authentication token used for API requests */
    token: string;
    /** The WebSocket URL for real-time connections */
    private url;
    /** The managed WebSocket connection */
    private socket;
    /** The base URL for HTTP API requests */
    private apiURL;
    /** Event topic that fires when the WebSocket connection is established */
    connected: Topic<void>;
    /** Event topic that fires when the WebSocket connection is lost */
    disconnected: Topic<void>;
    /** Event topic that fires when an error occurs */
    error: Topic<Error>;
    /**
     * All subscribable real-time events from the Stardust Relay
     *
     * @example Subscribe to chat messages
     * ```typescript
     * client.Events.Chat.new.subscribe((message) => {
     *   console.log(`${message.author.displayName}: ${message.content}`);
     * });
     * ```
     *
     * @example Subscribe to stream events
     * ```typescript
     * client.Events.Stream.follow.subscribe((event) => {
     *   console.log(`New follower: ${event.follower.displayName}`);
     * });
     * ```
     */
    Events: {
        /**
         * Chat events from supported platforms (Twitch, Discord, etc.)
         */
        Chat: {
            /** Fired when a new chat message is received */
            new: Topic<APIRelayMessage>;
            /** Fired when a chat message is deleted */
            deleted: Topic<APIRelayMessage>;
        };
        /**
         * Social media post events (Mastodon, custom prose, etc.)
         */
        Post: {
            /** Fired when a new post is created */
            new: Topic<APIRelayPost>;
        };
        /**
         * Streaming platform events, normalized from Twitch and other sources
         */
        Stream: {
            /** Fired when a stream goes live */
            live: Topic<APIStreamLiveEvent>;
            /** Fired when a stream request is made */
            request: Topic<APIStreamRequestEvent>;
            /** Fired when someone follows the stream */
            follow: Topic<APIStreamFollowEvent>;
            /** Fired when someone subscribes to the stream */
            subscription: Topic<APIStreamSubEvent>;
            /** Fired when someone gifts a subscription */
            giftedSubscription: Topic<APIStreamGiftedSubEvent>;
            /** Fired when someone gifts multiple subscriptions */
            massGiftedSubscription: Topic<APIStreamMassGiftedSubEvent>;
            /** Fired when the stream is raided */
            raid: Topic<APIStreamRaidEvent>;
            /** Fired when someone cheers with bits */
            cheer: Topic<APIStreamCheerEvent>;
        };
        /**
         * Custom events emitted by users or integrations
         */
        custom: Topic<APICustomCamperEvent<string, unknown>>;
    };
    /**
     * Create a new RelayClient instance
     *
     * @param token - Your authentication token from the Stardust Relay dashboard
     * @param configOverrides - Optional configuration overrides for custom deployments
     */
    constructor(token: string, configOverrides?: Partial<RelayClientConfig>);
    /**
     * Construct a qualified API route
     *
     * @param route - The route to construct, including the leading slash
     * @return The qualified API route
     *
     * @example
     * const resourceRoute = this.getAPIRoute("/resource/list");
     */
    private getAPIRoute;
    /**
     * Check if the WebSocket connection is currently active
     * @returns True if connected, false otherwise
     */
    get isConnected(): boolean;
    /**
     * Close the WebSocket connection to the Stardust Relay
     * This will stop all real-time event subscriptions
     */
    close(): void;
    /**
     * Get an array of posts made by a camper
     *
     * @param camperId - The ID of the camper to fetch posts for
     * @param source - Optional source to filter posts by (e.g., 'mastodon', 'prose')
     * @returns A promise which resolves to an array of posts
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get all posts by a camper
     * ```typescript
     * const posts = await client.getCamperPosts('camper-id');
     * console.log(`Found ${posts.length} posts`);
     * ```
     *
     * @example Get posts from a specific source
     * ```typescript
     * const mastodonPosts = await client.getCamperPosts('camper-id', 'mastodon');
     * ```
     */
    getCamperPosts(camperId: string, source?: PostService): Promise<APIRelayPost[]>;
    /**
     * Get an array of posts from a specific source
     *
     * @param source - The source to fetch posts from
     * @returns A promise which resolves to an array of posts from the specified source
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get all Mastodon posts
     * ```typescript
     * const mastodonPosts = await client.getPosts("mastodon");
     * mastodonPosts.forEach(post => console.log(post.content));
     * ```
     */
    getPosts(source: PostService): Promise<APIRelayPost[]>;
    /**
     * Get a single post by its ID
     *
     * @param postId - The unique ID of the post to fetch
     * @returns A promise which resolves to the post data
     * @throws {RelayAPIError} When the API request fails or post is not found
     *
     * @example
     * ```typescript
     * try {
     *   const post = await client.getPost('post-123');
     *   console.log('Post content:', post.content);
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 404) {
     *     console.log('Post not found');
     *   }
     * }
     * ```
     */
    getPost(postId: string): Promise<APIRelayPost>;
    /**
     * Get a single chat message by its ID
     *
     * @param chatId - The unique ID of the chat message to fetch
     * @returns A promise which resolves to the chat message data
     * @throws {RelayAPIError} When the API request fails or message is not found
     *
     * @example
     * ```typescript
     * const message = await client.getChat('message-123');
     * console.log(`${message.author.displayName}: ${message.content}`);
     * ```
     */
    getChat(chatId: string): Promise<APIRelayMessage>;
    /**
     * Fetch up to the most recent 100 messages from a chat log
     *
     * @param service - The chat service to fetch from (e.g., 'twitch', 'discord')
     * @param channel - The channel name or ID to fetch messages from
     * @returns A promise which resolves to an array of recent chat messages
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get Twitch chat history
     * ```typescript
     * const messages = await client.getChatLog('twitch', 'astralcamper');
     * console.log(`Retrieved ${messages.length} recent messages`);
     *
     * messages.forEach(msg => {
     *   console.log(`[${msg.timestamp}] ${msg.author.displayName}: ${msg.content}`);
     * });
     * ```
     */
    getChatLog(service: ChatService, channel: string): Promise<APIRelayMessage[]>;
    /**
     * Fetch the complete directory of all campers in the community
     *
     * @returns A promise which resolves to an array of all campers
     * @throws {RelayAPIError} When the API request fails
     *
     * @example
     * ```typescript
     * const campers = await client.getDirectory();
     * console.log(`Community has ${campers.length} campers`);
     *
     * // Find a specific camper
     * const camper = campers.find(c => c.handle === 'astralcamper');
     * if (camper) {
     *   console.log(`Found ${camper.displayName}`);
     * }
     * ```
     */
    getDirectory(): Promise<APICamper[]>;
    /**
     * Fetch a Camper's basic information by their handle
     * This returns a lightweight summary with core information
     *
     * @param handle - The unique handle of the camper (without @ symbol)
     * @returns A promise which resolves to the camper's basic information
     * @throws {RelayAPIError} When the API request fails or camper is not found
     *
     * @example
     * ```typescript
     * const camper = await client.getCamper('astralcamper');
     * console.log(`${camper.displayName} (@${camper.handle})`);
     * console.log(`Joined: ${camper.createdAt}`);
     * ```
     */
    getCamper(handle: string): Promise<APICamper>;
    /**
     * Fetch a Camper's complete profile by their handle
     * This contains extended information including bio, links, and preferences
     *
     * @param handle - The unique handle of the camper (without @ symbol)
     * @returns A promise which resolves to the camper's complete profile
     * @throws {RelayAPIError} When the API request fails or camper is not found
     *
     * @example
     * ```typescript
     * const profile = await client.getProfile('astralcamper');
     * console.log(`${profile.displayName} (@${profile.handle})`);
     * console.log(`Bio: ${profile.bio}`);
     * console.log(`Links: ${profile.links.map(l => l.url).join(', ')}`);
     * ```
     */
    getProfile(handle: string): Promise<APICamperProfile>;
    /**
     * Fetch the profile of the currently authenticated camper
     *
     * @returns A promise which resolves to your complete profile information
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const myProfile = await client.getMyProfile();
     * console.log(`Your display name: ${myProfile.displayName}`);
     * console.log(`Your handle: @${myProfile.handle}`);
     * console.log(`Account created: ${myProfile.createdAt}`);
     * ```
     */
    getMyProfile(): Promise<APICamperProfile>;
    /**
     * Fetch the most recent public notes for a specific camper or the whole community
     * Public notes are visible to everyone and often used for status updates
     *
     * @param handle - Optional handle of the camper to fetch notes for. If omitted, returns community notes
     * @returns A promise which resolves to an array of public notes
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get community notes
     * ```typescript
     * const communityNotes = await client.getPublicNotes();
     * console.log(`Latest community notes: ${communityNotes.length}`);
     * ```
     *
     * @example Get notes from a specific camper
     * ```typescript
     * const camperNotes = await client.getPublicNotes('astralcamper');
     * camperNotes.forEach(note => {
     *   console.log(`[${note.timestamp}] ${note.content}`);
     * });
     * ```
     */
    getPublicNotes(handle?: string): Promise<APINote[]>;
    /**
     * Fetch your private notes history
     * Private notes are only visible to you and can be used for personal reminders
     *
     * @returns A promise which resolves to an array of your private notes
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const myNotes = await client.getMyNotes();
     * console.log(`You have ${myNotes.length} private notes`);
     *
     * // Display recent notes
     * myNotes.slice(0, 5).forEach(note => {
     *   console.log(`[${note.timestamp}] ${note.content}`);
     * });
     * ```
     */
    getMyNotes(): Promise<APINote[]>;
    /**
     * Fetch your mailbox containing messages from other campers
     *
     * @returns A promise which resolves to an array of mail letters
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const mailbox = await client.getMailbox();
     * console.log(`You have ${mailbox.length} mail letters`);
     *
     * // Show unread messages
     * const unread = mailbox.filter(letter => !letter.read);
     * console.log(`${unread.length} unread messages`);
     *
     * unread.forEach(letter => {
     *   console.log(`From: ${letter.sender.displayName}`);
     *   console.log(`Subject: ${letter.subject}`);
     * });
     * ```
     */
    getMailbox(): Promise<APIMailLetter[]>;
    /**
     * Fetch the list of shortlinks created by you
     * Shortlinks are custom URLs that redirect to longer URLs
     *
     * @returns A promise which resolves to an array of your shortlinks
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const shortlinks = await client.getShortlinks();
     * console.log(`You have ${shortlinks.length} shortlinks`);
     *
     * shortlinks.forEach(link => {
     *   console.log(`${link.alias} -> ${link.href}`);
     *   console.log(`Created: ${link.createdAt}, Clicks: ${link.clicks}`);
     * });
     * ```
     */
    getShortlinks(): Promise<APIShortlink[]>;
    /**
     * Create a shortlink with the given alias and target URL
     *
     * @param href - The target URL to redirect to (must be a valid URL)
     * @param alias - Optional custom alias for the shortlink. If omitted, a random alias will be generated
     * @returns A promise which resolves to the created shortlink's alias
     * @throws {RelayAPIError} When the API request fails, alias is already taken, or URL is invalid
     *
     * @example Create a shortlink with auto-generated alias
     * ```typescript
     * const alias = await client.createShortlink('https://example.com');
     * console.log(`Shortlink created: https://astral.camp/s/${alias}`);
     * ```
     *
     * @example Create a shortlink with custom alias
     * ```typescript
     * try {
     *   const alias = await client.createShortlink('https://github.com/astralcamper/project', 'myproject');
     *   console.log(`Custom shortlink created: https://astral.camp/s/myproject`);
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 409) {
     *     console.log('Alias already taken, try a different one');
     *   }
     * }
     * ```
     */
    createShortlink(href: string, alias?: string): Promise<string>;
    /**
     * Delete a shortlink by its alias
     *
     * @param alias - The alias of the shortlink to delete
     * @returns A promise which resolves to a confirmation message when the shortlink is deleted
     * @throws {RelayAPIError} When the API request fails, shortlink not found, or you don't own the shortlink
     *
     * @example
     * ```typescript
     * try {
     *   const result = await client.deleteShortlink('myproject');
     *   console.log('Shortlink deleted successfully');
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 404) {
     *     console.log('Shortlink not found');
     *   } else if (error instanceof RelayAPIError && error.status === 403) {
     *     console.log('You do not own this shortlink');
     *   }
     * }
     * ```
     */
    deleteShortlink(alias: string): Promise<string>;
    /**
     * Send a message to the Twitch chat via the bot
     * The bot must be configured and active in the target channel
     *
     * @param message - The message to send to Twitch chat
     * @returns A promise which resolves when the message is sent successfully
     * @throws {RelayAPIError} When the API request fails, bot is not configured, or message is invalid
     *
     * @example
     * ```typescript
     * try {
     *   await client.twitchBotSay('Hello chat! This message was sent via the API.');
     *   console.log('Message sent to Twitch chat');
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 403) {
     *     console.log('Bot is not configured or not authorized');
     *   }
     * }
     * ```
     *
     * @example Send a dynamic message
     * ```typescript
     * const viewers = await getViewerCount(); // Your function
     * await client.twitchBotSay(`We currently have ${viewers} viewers! Welcome everyone!`);
     * ```
     */
    twitchBotSay(message: string): Promise<void>;
    /**
     * Emit a custom event to the Stardust Relay
     * This event will be broadcast to all connected clients for your account,
     * allowing you to create custom real-time integrations
     *
     * @param eventName - The name of the custom event to emit
     * @param data - Optional data to send with the event
     * @returns A promise which resolves when the event is emitted successfully
     * @throws {RelayAPIError} When the API request fails or event data is invalid
     *
     * @example Send a simple notification
     * ```typescript
     * await client.emit('user-action', {
     *   action: 'button-click',
     *   timestamp: new Date().toISOString(),
     *   userId: 'user123'
     * });
     * ```
     *
     * @example Send a game state update
     * ```typescript
     * await client.emit('game-state-change', {
     *   level: 5,
     *   score: 1500,
     *   powerUps: ['shield', 'speed-boost']
     * });
     * ```
     *
     * @example Listen for your custom events
     * ```typescript
     * // Subscribe to custom events
     * client.Events.custom.subscribe((event) => {
     *   if (event.eventName === 'user-action') {
     *     console.log('User action received:', event.eventData);
     *   }
     * });
     * ```
     */
    emit(eventName: string, data?: unknown): Promise<void>;
}

export { CAMPER_DEFAULT_AVATAR, CAMPER_DEFAULT_COLOR, CAMPER_DISPLAY_NAME_MAX_LENGTH, CAMPER_HANDLE_REGEX, CAMPER_TAGLINE_MAX_LENGTH, RelayAPIError, RelayClient as default };
export type { APICamperProfile as CamperProfile, ChatService, APICustomCamperEvent as RelayCustomEvent, APIRelayMessage as RelayMessage, APIRelayPost as RelayPost, APIStreamCheerEvent as StreamCheerEvent, APIStreamFollowEvent as StreamFollowEvent, APIStreamGiftedSubEvent as StreamGiftedSubEvent, APIStreamLiveEvent as StreamLiveEvent, APIStreamMassGiftedSubEvent as StreamMassGiftedSubEvent, APIStreamRaidEvent as StreamRaidEvent, APIStreamRequestEvent as StreamRequestEvent, APIStreamSubEvent as StreamSubEvent };
