// src/errors.ts
class SocketError extends Error {
  name = "SocketError";
  originalError;
  constructor(message, originalError) {
    super(message);
    this.originalError = originalError;
  }
}

class UnpackError extends SocketError {
}

// src/messages.ts
var Heartbeat = Symbol("Heartbeat");
function pack(data) {
  return JSON.stringify({ _: data });
}
function unpack(data) {
  try {
    const parsedData = JSON.parse(data);
    if (typeof parsedData === "number") {
      return Heartbeat;
    }
    if (typeof parsedData === "object" && parsedData !== null && "_" in parsedData && typeof parsedData._ !== "undefined") {
      return { data: parsedData._ };
    }
    throw new UnpackError("Unpack Error: Invalid message shape");
  } catch (error) {
    if (error instanceof Error) {
      throw new UnpackError("Unpack Error: Failed to deserialize", error);
    }
    throw error;
  }
}

// src/socket.ts
var THIRTY_SECONDS = 30 * 1000;

class ManagedSocket {
  url;
  socket;
  onMessage = () => {};
  onError = () => {};
  onConnected = () => {};
  onDisconnected = () => {};
  _reconnectAttempts = 0;
  _messageQueue = [];
  _debugMode = false;
  _offline = false;
  constructor(url, handlers = {}) {
    this.url = url;
    this.onMessage = handlers.onMessage ?? this.onMessage;
    this.onError = handlers.onError ?? this.onError;
    this.onConnected = handlers.onConnected ?? this.onConnected;
    this.onDisconnected = handlers.onDisconnected ?? this.onDisconnected;
    this.socket = this.connect();
    setInterval(() => {
      this.sendHeartbeat();
    }, THIRTY_SECONDS);
    window.addEventListener("offline", () => {
      this._offline = true;
      if (this.isConnected) {
        this.debugMessage("Internet connection lost, closing socket...");
        this.close();
      }
    });
    window.addEventListener("online", () => {
      this._offline = false;
      if (!this.isConnected) {
        this.debugMessage("Internet connection restored, reconnecting socket...");
        this.attemptReconnection();
      }
    });
  }
  sendHeartbeat() {
    if (this.socket?.readyState === WebSocket.OPEN) {
      this.socket.send(JSON.stringify(Date.now()));
    }
  }
  debugMessage(...args) {
    if (this._debugMode) {
      const output = ["[Socket]", ...args];
      console.log(...output);
    }
  }
  sendQueuedMessages() {
    const messagesToSend = [...this._messageQueue];
    this._messageQueue = [];
    for (const message of messagesToSend) {
      this.send(message);
    }
  }
  attemptReconnection() {
    if (this._offline) {
      this.debugMessage("Browser offline; skipping reconnection attempt");
      return;
    }
    const delay = Math.min(2000 ** this._reconnectAttempts, THIRTY_SECONDS);
    this.debugMessage(`Attempting to reconnect in ${delay}ms`);
    setTimeout(() => {
      this._reconnectAttempts++;
      this.connect();
    }, delay);
  }
  connect() {
    const newSocket = new WebSocket(this.url);
    newSocket.addEventListener("open", () => {
      this._reconnectAttempts = 0;
      this.onConnected();
      this.sendQueuedMessages();
    });
    newSocket.addEventListener("close", ({ wasClean, code, reason }) => {
      this.debugMessage("WebSocket closed", {
        wasClean,
        code,
        reason
      });
      this.onDisconnected({
        wasClean,
        code,
        reason
      });
      if (!this._offline) {
        this.debugMessage("Attempting automatic reconnection");
        return this.attemptReconnection();
      }
    });
    newSocket.addEventListener("error", (error) => {
      this.debugMessage("WebSocket error", error);
      const socketError = new SocketError("Socket error", new Error(error.currentTarget?.toString()));
      this.onError(socketError);
    });
    newSocket.addEventListener("message", (messageEvent) => {
      const data = unpack(messageEvent.data);
      if (data === Heartbeat) {
        return;
      }
      const { data: unpacked } = data;
      this.debugMessage("Received message", unpacked);
      this.onMessage(unpacked);
    });
    this.socket = newSocket;
    return newSocket;
  }
  get isConnected() {
    return this.socket?.readyState === WebSocket.OPEN;
  }
  debug(debugModeEnabled) {
    this._debugMode = debugModeEnabled;
  }
  close() {
    if (this.socket?.readyState !== WebSocket.OPEN) {
      throw new SocketError("Cannot close a socket that is not open");
    }
    this.socket.close();
  }
  send(data) {
    if (!this.socket || this.socket.readyState !== WebSocket.OPEN) {
      this.debugMessage("WebSocket is not open. Queuing message.");
      this._messageQueue.push(data);
      return;
    }
    this.socket.send(pack(data));
  }
}

/**
 * A simple pub-sub like topic system for easy event publishing and listening
 */
class Topic {
    beforeListeners = [];
    onListeners = [];
    afterListeners = [];
    removeSubscription = (handler, type) => {
        const isNotBeingRemoved = (subscription) => subscription.handler !== handler;
        switch (type) {
            case "before":
                this.beforeListeners = this.beforeListeners.filter(isNotBeingRemoved);
                break;
            case "on":
                this.onListeners = this.onListeners.filter(isNotBeingRemoved);
                break;
            case "after":
                this.afterListeners = this.afterListeners.filter(isNotBeingRemoved);
                break;
        }
    };
    createSubscription = (handler, type) => {
        return {
            handler,
            type,
            unsubscribe: () => {
                this.removeSubscription(handler, type);
            },
        };
    };
    /**
     * Register an event handler to run before normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    before = (handler) => {
        const subscription = this.createSubscription(handler, "before");
        this.beforeListeners.push(subscription);
        return subscription;
    };
    /**
     * Register an event handler
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    subscribe = (handler) => {
        const subscription = this.createSubscription(handler, "on");
        this.onListeners.push(subscription);
        return subscription;
    };
    /**
     * Register an event handler to run after normal handlers
     * @param handler A handler function which takes a payload and optional event name
     * @returns A subscription descriptor
     */
    after = (handler) => {
        const subscription = this.createSubscription(handler, "after");
        this.afterListeners.push(subscription);
        return subscription;
    };
    /**
     * Publish data and an optional event name ot the topic
     * @param payload Data to publish for this topic
     */
    publish = (payload) => {
        for (const { handler } of this.beforeListeners) {
            handler(payload);
        }
        for (const { handler } of this.onListeners) {
            handler(payload);
        }
        for (const { handler } of this.afterListeners) {
            handler(payload);
        }
    };
    /**
     * Automatically convert events from this topic and publish to another topic
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing
     */
    pipe = (topic, transformer) => {
        this.subscribe((payload) => {
            topic.publish(transformer(payload));
        });
    };
    /**
     * Automatically convert events from this topic and publish to another topic.
     * If the transformer returns void, the pipe will short-circuit instead.
     * @param topic The topic to pipe to
     * @param transformer A function to transform the payload before publishing or returns void to skip publishing
     */
    optionalPipe = (topic, check, transformer) => {
        this.subscribe((payload) => {
            if (check(payload)) {
                topic.publish(transformer(payload));
            }
        });
    };
}

const CAMPER_DEFAULT_AVATAR = "https://cdn.astral.camp/general-assets/default-avatar.png";
const CAMPER_DEFAULT_COLOR = "#FFFFFF";
const CAMPER_HANDLE_REGEX = /^[a-zA-Z0-9_-]{3,32}$/;
const CAMPER_DISPLAY_NAME_MAX_LENGTH = 32;
const CAMPER_TAGLINE_MAX_LENGTH = 120;
var CamperAccessLevel;
(function (CamperAccessLevel) {
    CamperAccessLevel[CamperAccessLevel["Banned"] = -1] = "Banned";
    CamperAccessLevel[CamperAccessLevel["Normal"] = 0] = "Normal";
    CamperAccessLevel[CamperAccessLevel["Firewatch"] = 1] = "Firewatch";
    CamperAccessLevel[CamperAccessLevel["Moderator"] = 2] = "Moderator";
    CamperAccessLevel[CamperAccessLevel["Admin"] = 3] = "Admin";
})(CamperAccessLevel || (CamperAccessLevel = {}));

/**
 * Error thrown when the Relay API returns an error response
 * Contains the HTTP status code and status text for debugging
 */
class RelayAPIError extends Error {
    /** The HTTP status code returned by the API */
    status;
    /** The HTTP status text returned by the API */
    statusText;
    constructor(status, statusText) {
        super(`[Relay API Error] ${status} ${statusText}`);
        this.status = status;
        this.statusText = statusText;
    }
}
/**
 * Make a GET request to the specified URL
 * @param url The complete URL to request
 * @param token The auth token to include with the request
 * @returns The response body as data
 * @throws {RelayAPIError} When the API returns an error response
 */
async function apiGet(url, token) {
    const res = await fetch(url, {
        method: "GET",
        headers: {
            Authorization: `Bearer ${token}`,
        },
    });
    if (res.ok) {
        return res.json();
    }
    throw new RelayAPIError(res.status, res.statusText);
}
/**
 * Make a DELETE request to the specified URL
 * @param url The complete URL to request
 * @param token The auth token to include with the request
 * @returns The response body as data
 * @throws {RelayAPIError} When the API returns an error response
 */
async function apiDelete(url, token) {
    const res = await fetch(url, {
        method: "DELETE",
        headers: {
            Authorization: `Bearer ${token}`,
        },
    });
    if (res.ok) {
        return res.json();
    }
    throw new RelayAPIError(res.status, res.statusText);
}
/**
 * Make a POST request to the specified URL
 * @param url The complete URL to request
 * @param token The auth token to include with the request
 * @param body The request body to send
 * @returns The response body as data
 * @throws {RelayAPIError} When the API returns an error response
 */
async function apiPost(url, token, body) {
    const res = await fetch(url, {
        method: "POST",
        headers: {
            Authorization: `Bearer ${token}`,
            "Content-Type": "application/json",
        },
        body: JSON.stringify(body),
    });
    if (res.ok) {
        return res.json();
    }
    throw new RelayAPIError(res.status, res.statusText);
}
const DEFAULT_CONFIG = {
    socketHost: "relay-ws.astral.camp",
    socketPort: 443,
    socketProtocol: "wss",
    socketPath: "/ws",
    socketQuery: "",
    httpHost: "relay.astral.camp",
    httpPort: 443,
    httpProtocol: "https",
};
/**
 * Client API for the Stardust Relay
 *
 * @example Usage
 * ```typescript
 * import RelayClient from '@stardust-relay/client';
 *
 * const client = new RelayClient("your-auth-token");
 *
 * // Subscribe to chat messages
 * client.Events.Chat.new.subscribe((message) => {
 *   console.log(message.author, message.content);
 * });
 *
 * // Fetch a camper's profile
 * const profile = await client.getProfile('username');
 *
 * // Send a custom event
 * await client.emit("namespace:event", { key: "value" });
 * ```
 */
class RelayClient {
    /** The authentication token used for API requests */
    token;
    /** The WebSocket URL for real-time connections */
    url = "https://relay-ws.astral.camp/ws";
    /** The managed WebSocket connection */
    socket;
    /** The base URL for HTTP API requests */
    apiURL;
    /** Event topic that fires when the WebSocket connection is established */
    connected = new Topic();
    /** Event topic that fires when the WebSocket connection is lost */
    disconnected = new Topic();
    /** Event topic that fires when an error occurs */
    error = new Topic();
    /**
     * All subscribable real-time events from the Stardust Relay
     *
     * @example Subscribe to chat messages
     * ```typescript
     * client.Events.Chat.new.subscribe((message) => {
     *   console.log(`${message.author.displayName}: ${message.content}`);
     * });
     * ```
     *
     * @example Subscribe to stream events
     * ```typescript
     * client.Events.Stream.follow.subscribe((event) => {
     *   console.log(`New follower: ${event.follower.displayName}`);
     * });
     * ```
     */
    Events = {
        /**
         * Chat events from supported platforms (Twitch, Discord, etc.)
         */
        Chat: {
            /** Fired when a new chat message is received */
            new: new Topic(),
            /** Fired when a chat message is deleted */
            deleted: new Topic(),
        },
        /**
         * Social media post events (Mastodon, custom prose, etc.)
         */
        Post: {
            /** Fired when a new post is created */
            new: new Topic(),
        },
        /**
         * Streaming platform events, normalized from Twitch and other sources
         */
        Stream: {
            /** Fired when a stream goes live */
            live: new Topic(),
            /** Fired when a stream request is made */
            request: new Topic(),
            /** Fired when someone follows the stream */
            follow: new Topic(),
            /** Fired when someone subscribes to the stream */
            subscription: new Topic(),
            /** Fired when someone gifts a subscription */
            giftedSubscription: new Topic(),
            /** Fired when someone gifts multiple subscriptions */
            massGiftedSubscription: new Topic(),
            /** Fired when the stream is raided */
            raid: new Topic(),
            /** Fired when someone cheers with bits */
            cheer: new Topic(),
        },
        /**
         * Custom events emitted by users or integrations
         */
        custom: new Topic(),
    };
    /**
     * Create a new RelayClient instance
     *
     * @param token - Your authentication token from the Stardust Relay dashboard
     * @param configOverrides - Optional configuration overrides for custom deployments
     */
    constructor(token, configOverrides) {
        const { socketHost, socketPort, socketProtocol, socketPath, socketQuery, httpHost, httpPort, httpProtocol, } = Object.assign({}, DEFAULT_CONFIG, configOverrides);
        this.token = token;
        const socketConnURL = new URL(`${socketProtocol}://${socketHost}:${socketPort}${socketPath}`);
        socketConnURL.search = socketQuery;
        socketConnURL.searchParams.set("token", token);
        this.url = socketConnURL.toString();
        this.apiURL = new URL(`${httpProtocol}://${httpHost}:${httpPort}`).toString();
        this.socket = new ManagedSocket(this.url, {
            onConnected: () => {
                this.connected.publish();
            },
            onDisconnected: () => {
                this.disconnected.publish();
            },
            onError: (error) => {
                this.error.publish(error);
            },
            onMessage: (raw) => {
                const message = raw;
                switch (message.event) {
                    case "chat:message:created":
                        return this.Events.Chat.new.publish(message.data);
                    case "chat:message:deleted":
                        return this.Events.Chat.deleted.publish(message.data);
                    case "post:created":
                        return this.Events.Post.new.publish(message.data);
                    case "stream:live":
                        return this.Events.Stream.live.publish(message.data);
                    case "stream:request":
                        return this.Events.Stream.request.publish(message.data);
                    case "stream:follow":
                        return this.Events.Stream.follow.publish(message.data);
                    case "stream:subscription":
                        return this.Events.Stream.subscription.publish(message.data);
                    case "stream:gifted-subscription":
                        return this.Events.Stream.giftedSubscription.publish(message.data);
                    case "stream:mass-gifted-subscription":
                        return this.Events.Stream.massGiftedSubscription.publish(message.data);
                    case "stream:raid":
                        return this.Events.Stream.raid.publish(message.data);
                    case "stream:cheer":
                        return this.Events.Stream.cheer.publish(message.data);
                    case "relay:custom-event":
                        return this.Events.custom.publish(message.data);
                }
            },
        });
    }
    /**
     * Construct a qualified API route
     *
     * @param route - The route to construct, including the leading slash
     * @return The qualified API route
     *
     * @example
     * const resourceRoute = this.getAPIRoute("/resource/list");
     */
    getAPIRoute(route) {
        return new URL(route, this.apiURL).toString();
    }
    /**
     * Check if the WebSocket connection is currently active
     * @returns True if connected, false otherwise
     */
    get isConnected() {
        return this.socket.isConnected;
    }
    /**
     * Close the WebSocket connection to the Stardust Relay
     * This will stop all real-time event subscriptions
     */
    close() {
        this.socket.close();
    }
    /**
     * Get an array of posts made by a camper
     *
     * @param camperId - The ID of the camper to fetch posts for
     * @param source - Optional source to filter posts by (e.g., 'mastodon', 'prose')
     * @returns A promise which resolves to an array of posts
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get all posts by a camper
     * ```typescript
     * const posts = await client.getCamperPosts('camper-id');
     * console.log(`Found ${posts.length} posts`);
     * ```
     *
     * @example Get posts from a specific source
     * ```typescript
     * const mastodonPosts = await client.getCamperPosts('camper-id', 'mastodon');
     * ```
     */
    async getCamperPosts(camperId, source) {
        if (source) {
            return apiGet(this.getAPIRoute(`/post/collection/${camperId}/${source}`), this.token);
        }
        return apiGet(this.getAPIRoute(`/post/collection/camper/${camperId}`), this.token);
    }
    /**
     * Get an array of posts from a specific source
     *
     * @param source - The source to fetch posts from
     * @returns A promise which resolves to an array of posts from the specified source
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get all Mastodon posts
     * ```typescript
     * const mastodonPosts = await client.getPosts("mastodon");
     * mastodonPosts.forEach(post => console.log(post.content));
     * ```
     */
    async getPosts(source) {
        return apiGet(this.getAPIRoute(`/post/collection/source/${source}`), this.token);
    }
    /**
     * Get a single post by its ID
     *
     * @param postId - The unique ID of the post to fetch
     * @returns A promise which resolves to the post data
     * @throws {RelayAPIError} When the API request fails or post is not found
     *
     * @example
     * ```typescript
     * try {
     *   const post = await client.getPost('post-123');
     *   console.log('Post content:', post.content);
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 404) {
     *     console.log('Post not found');
     *   }
     * }
     * ```
     */
    async getPost(postId) {
        return apiGet(this.getAPIRoute(`/post/${postId}`), this.token);
    }
    /**
     * Get a single chat message by its ID
     *
     * @param chatId - The unique ID of the chat message to fetch
     * @returns A promise which resolves to the chat message data
     * @throws {RelayAPIError} When the API request fails or message is not found
     *
     * @example
     * ```typescript
     * const message = await client.getChat('message-123');
     * console.log(`${message.author.displayName}: ${message.content}`);
     * ```
     */
    async getChat(chatId) {
        return apiGet(this.getAPIRoute(`/chat/${chatId}`), this.token);
    }
    /**
     * Fetch up to the most recent 100 messages from a chat log
     *
     * @param service - The chat service to fetch from (e.g., 'twitch', 'discord')
     * @param channel - The channel name or ID to fetch messages from
     * @returns A promise which resolves to an array of recent chat messages
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get Twitch chat history
     * ```typescript
     * const messages = await client.getChatLog('twitch', 'astralcamper');
     * console.log(`Retrieved ${messages.length} recent messages`);
     *
     * messages.forEach(msg => {
     *   console.log(`[${msg.timestamp}] ${msg.author.displayName}: ${msg.content}`);
     * });
     * ```
     */
    async getChatLog(service, channel) {
        return apiGet(this.getAPIRoute(`/chat/log/${service}/${channel}`), this.token);
    }
    /**
     * Fetch the complete directory of all campers in the community
     *
     * @returns A promise which resolves to an array of all campers
     * @throws {RelayAPIError} When the API request fails
     *
     * @example
     * ```typescript
     * const campers = await client.getDirectory();
     * console.log(`Community has ${campers.length} campers`);
     *
     * // Find a specific camper
     * const camper = campers.find(c => c.handle === 'astralcamper');
     * if (camper) {
     *   console.log(`Found ${camper.displayName}`);
     * }
     * ```
     */
    async getDirectory() {
        return await apiGet(this.getAPIRoute("/profile/directory"), this.token);
    }
    /**
     * Fetch a Camper's basic information by their handle
     * This returns a lightweight summary with core information
     *
     * @param handle - The unique handle of the camper (without @ symbol)
     * @returns A promise which resolves to the camper's basic information
     * @throws {RelayAPIError} When the API request fails or camper is not found
     *
     * @example
     * ```typescript
     * const camper = await client.getCamper('astralcamper');
     * console.log(`${camper.displayName} (@${camper.handle})`);
     * console.log(`Joined: ${camper.createdAt}`);
     * ```
     */
    async getCamper(handle) {
        return await apiGet(this.getAPIRoute(`/profile/${handle}/summary`), this.token);
    }
    /**
     * Fetch a Camper's complete profile by their handle
     * This contains extended information including bio, links, and preferences
     *
     * @param handle - The unique handle of the camper (without @ symbol)
     * @returns A promise which resolves to the camper's complete profile
     * @throws {RelayAPIError} When the API request fails or camper is not found
     *
     * @example
     * ```typescript
     * const profile = await client.getProfile('astralcamper');
     * console.log(`${profile.displayName} (@${profile.handle})`);
     * console.log(`Bio: ${profile.bio}`);
     * console.log(`Links: ${profile.links.map(l => l.url).join(', ')}`);
     * ```
     */
    async getProfile(handle) {
        return await apiGet(this.getAPIRoute(`/profile/${handle}`), this.token);
    }
    /**
     * Fetch the profile of the currently authenticated camper
     *
     * @returns A promise which resolves to your complete profile information
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const myProfile = await client.getMyProfile();
     * console.log(`Your display name: ${myProfile.displayName}`);
     * console.log(`Your handle: @${myProfile.handle}`);
     * console.log(`Account created: ${myProfile.createdAt}`);
     * ```
     */
    async getMyProfile() {
        return await apiGet(this.getAPIRoute("/profile"), this.token);
    }
    /**
     * Fetch the most recent public notes for a specific camper or the whole community
     * Public notes are visible to everyone and often used for status updates
     *
     * @param handle - Optional handle of the camper to fetch notes for. If omitted, returns community notes
     * @returns A promise which resolves to an array of public notes
     * @throws {RelayAPIError} When the API request fails
     *
     * @example Get community notes
     * ```typescript
     * const communityNotes = await client.getPublicNotes();
     * console.log(`Latest community notes: ${communityNotes.length}`);
     * ```
     *
     * @example Get notes from a specific camper
     * ```typescript
     * const camperNotes = await client.getPublicNotes('astralcamper');
     * camperNotes.forEach(note => {
     *   console.log(`[${note.timestamp}] ${note.content}`);
     * });
     * ```
     */
    async getPublicNotes(handle) {
        if (handle) {
            return apiGet(this.getAPIRoute(`/note/public/${handle}`), this.token);
        }
        return apiGet(this.getAPIRoute("/note/public"), this.token);
    }
    /**
     * Fetch your private notes history
     * Private notes are only visible to you and can be used for personal reminders
     *
     * @returns A promise which resolves to an array of your private notes
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const myNotes = await client.getMyNotes();
     * console.log(`You have ${myNotes.length} private notes`);
     *
     * // Display recent notes
     * myNotes.slice(0, 5).forEach(note => {
     *   console.log(`[${note.timestamp}] ${note.content}`);
     * });
     * ```
     */
    async getMyNotes() {
        return apiGet(this.getAPIRoute("/note"), this.token);
    }
    /**
     * Fetch your mailbox containing messages from other campers
     *
     * @returns A promise which resolves to an array of mail letters
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const mailbox = await client.getMailbox();
     * console.log(`You have ${mailbox.length} mail letters`);
     *
     * // Show unread messages
     * const unread = mailbox.filter(letter => !letter.read);
     * console.log(`${unread.length} unread messages`);
     *
     * unread.forEach(letter => {
     *   console.log(`From: ${letter.sender.displayName}`);
     *   console.log(`Subject: ${letter.subject}`);
     * });
     * ```
     */
    async getMailbox() {
        return apiGet(this.getAPIRoute("/mail"), this.token);
    }
    /**
     * Fetch the list of shortlinks created by you
     * Shortlinks are custom URLs that redirect to longer URLs
     *
     * @returns A promise which resolves to an array of your shortlinks
     * @throws {RelayAPIError} When the API request fails or token is invalid
     *
     * @example
     * ```typescript
     * const shortlinks = await client.getShortlinks();
     * console.log(`You have ${shortlinks.length} shortlinks`);
     *
     * shortlinks.forEach(link => {
     *   console.log(`${link.alias} -> ${link.href}`);
     *   console.log(`Created: ${link.createdAt}, Clicks: ${link.clicks}`);
     * });
     * ```
     */
    async getShortlinks() {
        return apiGet(this.getAPIRoute("/shortlink"), this.token);
    }
    // == API METHODS ==
    /**
     * Create a shortlink with the given alias and target URL
     *
     * @param href - The target URL to redirect to (must be a valid URL)
     * @param alias - Optional custom alias for the shortlink. If omitted, a random alias will be generated
     * @returns A promise which resolves to the created shortlink's alias
     * @throws {RelayAPIError} When the API request fails, alias is already taken, or URL is invalid
     *
     * @example Create a shortlink with auto-generated alias
     * ```typescript
     * const alias = await client.createShortlink('https://example.com');
     * console.log(`Shortlink created: https://astral.camp/s/${alias}`);
     * ```
     *
     * @example Create a shortlink with custom alias
     * ```typescript
     * try {
     *   const alias = await client.createShortlink('https://github.com/astralcamper/project', 'myproject');
     *   console.log(`Custom shortlink created: https://astral.camp/s/myproject`);
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 409) {
     *     console.log('Alias already taken, try a different one');
     *   }
     * }
     * ```
     */
    async createShortlink(href, alias) {
        return apiPost(this.getAPIRoute("/shortlink"), this.token, { href, alias });
    }
    /**
     * Delete a shortlink by its alias
     *
     * @param alias - The alias of the shortlink to delete
     * @returns A promise which resolves to a confirmation message when the shortlink is deleted
     * @throws {RelayAPIError} When the API request fails, shortlink not found, or you don't own the shortlink
     *
     * @example
     * ```typescript
     * try {
     *   const result = await client.deleteShortlink('myproject');
     *   console.log('Shortlink deleted successfully');
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 404) {
     *     console.log('Shortlink not found');
     *   } else if (error instanceof RelayAPIError && error.status === 403) {
     *     console.log('You do not own this shortlink');
     *   }
     * }
     * ```
     */
    async deleteShortlink(alias) {
        return apiDelete(this.getAPIRoute(`/shortlink/${alias}`), this.token);
    }
    /**
     * Send a message to the Twitch chat via the bot
     * The bot must be configured and active in the target channel
     *
     * @param message - The message to send to Twitch chat
     * @returns A promise which resolves when the message is sent successfully
     * @throws {RelayAPIError} When the API request fails, bot is not configured, or message is invalid
     *
     * @example
     * ```typescript
     * try {
     *   await client.twitchBotSay('Hello chat! This message was sent via the API.');
     *   console.log('Message sent to Twitch chat');
     * } catch (error) {
     *   if (error instanceof RelayAPIError && error.status === 403) {
     *     console.log('Bot is not configured or not authorized');
     *   }
     * }
     * ```
     *
     * @example Send a dynamic message
     * ```typescript
     * const viewers = await getViewerCount(); // Your function
     * await client.twitchBotSay(`We currently have ${viewers} viewers! Welcome everyone!`);
     * ```
     */
    async twitchBotSay(message) {
        return apiPost(this.getAPIRoute("/twitch/bot/say"), this.token, { message });
    }
    /**
     * Emit a custom event to the Stardust Relay
     * This event will be broadcast to all connected clients for your account,
     * allowing you to create custom real-time integrations
     *
     * @param eventName - The name of the custom event to emit
     * @param data - Optional data to send with the event
     * @returns A promise which resolves when the event is emitted successfully
     * @throws {RelayAPIError} When the API request fails or event data is invalid
     *
     * @example Send a simple notification
     * ```typescript
     * await client.emit('user-action', {
     *   action: 'button-click',
     *   timestamp: new Date().toISOString(),
     *   userId: 'user123'
     * });
     * ```
     *
     * @example Send a game state update
     * ```typescript
     * await client.emit('game-state-change', {
     *   level: 5,
     *   score: 1500,
     *   powerUps: ['shield', 'speed-boost']
     * });
     * ```
     *
     * @example Listen for your custom events
     * ```typescript
     * // Subscribe to custom events
     * client.Events.custom.subscribe((event) => {
     *   if (event.eventName === 'user-action') {
     *     console.log('User action received:', event.eventData);
     *   }
     * });
     * ```
     */
    emit(eventName, data) {
        return apiPost(this.getAPIRoute("/event/emit"), this.token, {
            eventName,
            eventData: data,
        });
    }
}

export { CAMPER_DEFAULT_AVATAR, CAMPER_DEFAULT_COLOR, CAMPER_DISPLAY_NAME_MAX_LENGTH, CAMPER_HANDLE_REGEX, CAMPER_TAGLINE_MAX_LENGTH, RelayAPIError, RelayClient as default };
